@@ -492,7 +492,7 @@ public:
                     }
                     break;
                 }
-                else --_buf->refcnt;
+                else --_buf->refcnt; // fallthrough
             case State::LITERAL:
                 auto old_str = _str;
                 _new_auto(_length);
@@ -775,11 +775,13 @@ public:
 
     template <class Alloc2>
     basic_string& append (const basic_string<CharT, Traits, Alloc2>& str) {
+        if (!_length && _state != State::INTERNAL) return assign(str); // do not optimize if string had reserved memory
         return append(str._str, str._length);
     }
 
     template <class Alloc2>
     basic_string& append (const basic_string<CharT, Traits, Alloc2>& str, size_type pos, size_type count = npos) {
+        if (!_length && _state != State::INTERNAL) return assign(str, pos, count); // do not optimize if string had reserved memory
         if (pos > str._length) throw std::out_of_range("basic_string::append");
         if (count > str._length - pos) count = str._length - pos;
         if (count) { // can't call append(const CharT*, size_type) because otherwise if &str == this a fuckup would occur
